#****************************************************************************
#**
#**  File     :  /lua/HandicapVotingSim.lua
#**  Author(s):  D10ny5u5
#**
#**  Summary  :  Handicap Mod for Forged Alliance
#**
#****************************************************************************
local modPath = 'HandicapVoting'

local cur = {
	innocents = {},
	handicaps = {},
	handicapIndexes = {},
	allPlayers = {},
	VotedIndexes = {},
	delta = { mass = {}, energy = {} }, 
	handicap_war = false,
	trash = nil,
	config = {
		vote = 1 * 20,
		declare = 2 * 20,
		handicap_coefficient = 1/3, 
		handicap_economy = {}
	},
	handicap_bonus = { mass = 0, energy = 0 },
	stats = {
		handicap_count = 0,
		innocent_count = 0,
		allPlayers_count = 0,
		handicaps_dead = 0,
		innocents_dead = 0
	},
	votes = {},
	mass = {},
	energy = {}
}


function HandicapMainThread()
	#cur.config.handicap_economy[0] = { amount = 0.0001 }
	cur.config.handicap_economy[1] = { amount = 0.250 }
	cur.votes[1] = 0
	cur.votes[2] = 0
	cur.votes[3] = 0
	cur.votes[4] = 0
	cur.votes[5] = 0
	cur.votes[6] = 0
	cur.votes[7] = 0
	cur.votes[8] = 0
	intVotes = 0
	
	cur.trash = TrashBag()

	import('/lua/SimPlayerQuery.lua').AddQueryListener("HandicapVote", vote)

	Sync.pSkin = true
	
		
	Sync.PAssignment = "Handicap Assignment Pending"
	WaitSeconds(cur.config.vote)
	Sync.pVote = true
	WaitSeconds(cur.config.declare - cur.config.vote)

	# Move all elegiable players to the innocents table
	for army, brain in ArmyBrains do
		if ArmyIsCivilian(army) == false and ArmyIsOutOfGame(army) == false then
			table.insert(cur.innocents, army)
		end
	end

	# Move all elegiable players to the allPlayers table
	for army, brain in ArmyBrains do
		if ArmyIsCivilian(army) == false and ArmyIsOutOfGame(army) == false then
			table.insert(cur.allPlayers, army)
		end
	end
	
	local handicapCount = 0
	for i in cur.votes do
	
		# Did they get a vote
		if cur.votes[i] > 0 then
		
			# Was it one vote for themselves
			if cur.VotedIndexes[i] == i then
				LOG("Voted for yourself: ", 0)
					
				########################################################### Next line is commented for debugging only
				cur.votes[i] = cur.votes[i] - 1
			end
			
			if cur.votes[i] > 0 then
				LOG("Moving: ", i)
				handicapCount = handicapCount + 1
			end
			
		end
	end
	
	cur.stats.handicap_count = handicapCount
	cur.stats.innocent_count = table.getn(cur.innocents) - cur.stats.handicap_count
	cur.stats.allPlayers_count = cur.stats.handicap_count + cur.stats.innocent_count
	
	LOG("Number of Players: ", cur.stats.allPlayers_count)
	
	# Add handicaps and remember their original index
	for index, army in cur.innocents do
		if cur.votes[index] > 0 then
			table.insert(cur.handicaps, cur.innocents[index])
			table.insert(cur.handicapIndexes, index)
		end
	end
	
	#Debug
	for handicapIndex, allPlayersIndex in cur.handicapIndexes do
		LOG("Handicap Index: ", handicapIndex)
		LOG("allPlayers Index: ", allPlayersIndex)
	end
	
	# Remove handicaps from the innocents table
	for i=1,table.getn(cur.innocents) do
		for index, army in cur.handicaps do
			if cur.innocents[i] == army then
				cur.innocents[i] = nil
			end
		end
	end
	
	# Prepare data to send to the UI
	pData = {
		handicap_armies = {}, 
		innocent_armies = {}, 
		isHandicap = false
	}
	
	if IsHandicap(GetFocusArmy()) then
		LOG("Votes: ", NumVotes(GetFocusArmy()))
		intVotes = NumVotes(GetFocusArmy())
		
		if intVotes == 1 then
			Sync.pAlert = { "The votes are in: Handicap", "You get single bonus " }
		else
			Sync.pAlert = { "The votes are in: Handicap", "You get "..intVotes.."x bonus" }
		end
		
		Sync.PAssignment = "Handicap x "..intVotes
		pData.isHandicap = true
		
	else
		Sync.pAlert = { "The votes are in: ", "No bonus for you" }
		Sync.PAssignment = "No Bonus"
		pData.isHandicap = false
	end
	
	pData.handicap_armies = cur.handicaps
	pData.innocent_armies = cur.innocents
	Sync.pData = pData
	
	LOG("Inside HandicapVotingSim > HandicapMainThread - Before ForkThread(HandicapResourceThread)", 0)
	
	# Fire up the eco thread
	cur.trash:Add(ForkThread(HandicapResourceThread))
	
	# Main loop
	local end_game = false
	local stats_changed = true
	while end_game == false do
		WaitSeconds(0.1)

		local innocents_dead = 0
		local handicaps_dead = 0
		for index, army in cur.handicaps do
			if ArmyIsOutOfGame(army) then
				handicaps_dead = handicaps_dead + 1
			end
		end
		for index, army in cur.innocents do
			if ArmyIsOutOfGame(army) then
				innocents_dead = innocents_dead + 1
			end
		end

	end
	cur.trash:Destroy()
end

# This functions gives the resource bonus to the handicaps
function HandicapResourceThread()
	pEco = {
		mass = 0,
		energy = 0
	}
	while true do
		WaitSeconds(0.1)
		
		# Work out players current income
		local mass = 0
		local energy = 0
		
		for index, army in cur.allPlayers do
			
			if ArmyIsOutOfGame(army) == false then
				
				brain = ArmyBrains[army]
				
				cur.mass[index] = brain:GetEconomyIncome('MASS')
				cur.energy[index] = brain:GetEconomyIncome('ENERGY')
				
			end
		end
			
		# Give bonus to handicaps		
		for indexHandicaps, armyHandicaps in cur.handicaps do
			if ArmyIsOutOfGame(armyHandicaps) == false then
			
				local index1 = cur.handicapIndexes[indexHandicaps]
				
				local player_coeffiecient = cur.config.handicap_economy[1].amount							
			
				#Multiply by number of votes / number of players
				#player_coeffiecient = player_coeffiecient * NumVotes(index1) / cur.stats.allPlayers_count
				player_coeffiecient = NumVotes(index1) / 10
				
				player_mass = (cur.mass[index1] * player_coeffiecient)
				player_energy = (cur.energy[index1] * player_coeffiecient)
	
				LOG("Player Index: ", index1)
				LOG("Adding mass: ", player_mass)
				
				brain = ArmyBrains[armyHandicaps]			
				brain:GiveResource('MASS', player_mass)
				brain:GiveResource('ENERGY', player_energy)

				#Remove bonus from people who voted
				
				#Search through votes indexes for players that voted for this player
				for voteIndex, vote in cur.VotedIndexes do
				
					LOG("Looking at voted index: ", voteIndex)
					LOG("Voted for: ", vote)
					
					# Did they vote for themselves?  This is taken care of by votes
					if voteIndex == vote then
						LOG("Move along, nothing to see here ", 0)
					elseif vote == index1 then
					
						LOG("Found something: ", cur.VotedIndexes[voteIndex])
						LOG("What about me: ", cur.allPlayers[voteIndex])
						LOG("Removing Mass: ", player_mass)
						
						
						brainFrom = ArmyBrains[cur.allPlayers[voteIndex]]	
					
						brainFrom:TakeResource('MASS', player_mass)
						brainFrom:TakeResource('ENERGY', player_energy)
						
					end
				end
				
				# Update UI
				if armyHandicaps == GetFocusArmy() then
					# Sync with UI
					pEco.mass = math.ceil(player_mass * 10)
					pEco.energy = math.ceil(player_energy * 10)
					Sync.pEco = pEco
				end
			
			end
		end
	end
end

# Determines if an army is a handicap
function IsHandicap(army)

	for index, p in cur.allPlayers do
		if p == army then		
			if cur.votes[cur.allPlayers[index]] > 0 then
				LOG("Current player is IsHandicap: ", 1)
				return true
			end
		end
	end	
	LOG("Not IsHandicap", army)
	return false
end

# Return number of votes for an army
function NumVotes(army)
	for index, p in cur.allPlayers do
		if p == army then		
			return cur.votes[cur.allPlayers[index]]
		end
	end	
	LOG("No NumVotes for current player, this should never happen", 0)
	return 0
end

# Vote (from UI)
function vote(data)
	
	cur.votes[data.Args.Vote] = cur.votes[data.Args.Vote] + 1
	LOG("Vote To: ", data.Args.Vote)
	LOG("Vote From: ", data.Args.From)
	cur.VotedIndexes[data.Args.From] = data.Args.Vote
	print(data.Args.Nick.." voted for "..data.Args.VoteNick)
	
end