#****************************************************************************
#**
#**  File     :  /modules/HandicapVoting.lua
#**  Author(s):  D10ny5u5
#**
#**  Summary  :  Handicap Mod for Forged Alliance
#**
#****************************************************************************
local modPath = 'HandicapVoting'

local UIUtil = import('/lua/ui/uiutil.lua')
local LayoutHelpers = import('/lua/maui/layouthelpers.lua')
local Group = import('/lua/maui/group.lua').Group
local Bitmap = import('/lua/maui/bitmap.lua').Bitmap
local Button = import('/lua/maui/button.lua').Button
local Checkbox = import('/lua/maui/checkbox.lua').Checkbox
local StatusBar = import('/lua/maui/statusbar.lua').StatusBar
local GameMain = import('/lua/ui/game/gamemain.lua')
local Tooltip = import('/lua/ui/game/tooltip.lua')
local Prefs = import('/lua/user/prefs.lua')
local Tooltip = import('/lua/ui/game/tooltip.lua')

	# How long until handicap is assigned - this is just the timer display I think
local declare = 2 * 20
local voteDialog = nil

pUI = {
	arrow = false,
	box = false
}

pData = {
	handicap_armies = {},
	innocent_armies = {},
	isHandicap = false
}

pStats = {
	handicap_count = 0,
	innocent_count = 0,
	handicaps_dead = 0,
	innocents_dead = 0
}

parent = false
function CreateModUI(isReplay, _parent)
	parent = _parent
	
	BuildUI()
	SetLayout()
	CommonLogic()
	
	LOG("Finished HandicapVoting > CreateModUI", 0)
	
end

function BuildUI()
	# Create arrow checkbox
	pUI.arrow = Checkbox(parent)

	# Create group for main UI
	pUI.box = Group(parent)
	
	# Create main UI objects
	pUI.box.panel = Bitmap(pUI.box)
	pUI.box.leftBracket = Bitmap(pUI.box)
	pUI.box.leftBracketGlow = Bitmap(pUI.box)

	pUI.box.rightGlowTop = Bitmap(pUI.box)
	pUI.box.rightGlowMiddle = Bitmap(pUI.box)
	pUI.box.rightGlowBottom = Bitmap(pUI.box)
	
	pUI.box.title = UIUtil.CreateText(pUI.box, '', 18, UIUtil.bodyFont)
	pUI.box.title:SetDropShadow(true)
	pUI.box.countdown = UIUtil.CreateText(pUI.box, '', 14, UIUtil.bodyFont)
	pUI.box.countdown:SetDropShadow(true)
	
	pUI.box.mass = UIUtil.CreateText(pUI.box, '+0', 10, UIUtil.bodyFont)
	pUI.box.mass:SetDropShadow(true)
	pUI.box.energy = UIUtil.CreateText(pUI.box, '+0', 10, UIUtil.bodyFont)
	pUI.box.energy:SetDropShadow(true)

	pUI.box.kills = UIUtil.CreateText(pUI.box, '', 14, UIUtil.bodyFont)
	pUI.box.kills:SetDropShadow(true)
	
	pUI.box.killIcon = Bitmap(pUI.box)
	
	pUI.box.massBtn = Button(pUI.box,
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/mass_btn_up_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/mass_btn_down_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/mass_btn_over_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/mass_btn_dis_ph.dds'), 
		'', '')

	pUI.box.energyBtn = Button(pUI.box,
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/energy_btn_up_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/energy_btn_down_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/energy_btn_over_ph.dds'),
		UIUtil.SkinnableFile('/mods/'..modPath..'/textures/resources/energy_btn_dis_ph.dds'), 
		'', '')	

	LOG("Finished HandicapVoting > BuildUI", 0)		
end

function SetLayout()
	# Assign layout info to arrow checkbox
	pUI.arrow:SetTexture(UIUtil.UIFile('/game/tab-l-btn/tab-close_btn_up.dds'))
	pUI.arrow:SetNewTextures(UIUtil.UIFile('/game/tab-l-btn/tab-close_btn_up.dds'),
		UIUtil.UIFile('/game/tab-l-btn/tab-open_btn_up.dds'),
		UIUtil.UIFile('/game/tab-l-btn/tab-close_btn_over.dds'),
		UIUtil.UIFile('/game/tab-l-btn/tab-open_btn_over.dds'),
		UIUtil.UIFile('/game/tab-l-btn/tab-close_btn_dis.dds'),
		UIUtil.UIFile('/game/tab-l-btn/tab-open_btn_dis.dds'))
		
	LayoutHelpers.AtLeftTopIn(pUI.arrow, GetFrame(0), -3, 172)
	pUI.arrow.Depth:Set(function() return pUI.box.Depth() + 10 end)

	# Assign layout info to main UI
	pUI.box.panel:SetTexture(UIUtil.UIFile('/game/resource-panel/resources_panel_bmp.dds'))
	LayoutHelpers.AtLeftTopIn(pUI.box.panel, pUI.box)

	pUI.box.Height:Set(pUI.box.panel.Height)
	pUI.box.Width:Set(pUI.box.panel.Width)
	LayoutHelpers.AtLeftTopIn(pUI.box, parent, 16, 153)
	
	pUI.box:DisableHitTest()

	pUI.box.leftBracket:SetTexture(UIUtil.UIFile('/game/filter-ping-panel/bracket-left_bmp.dds'))
	pUI.box.leftBracketGlow:SetTexture(UIUtil.UIFile('/game/filter-ping-panel/bracket-energy-l_bmp.dds'))

	pUI.box.leftBracket.Right:Set(function() return pUI.box.panel.Left() + 10 end)
	pUI.box.leftBracketGlow.Left:Set(function() return pUI.box.leftBracket.Left() + 12 end)

	pUI.box.leftBracket.Depth:Set(pUI.box.panel.Depth)
	pUI.box.leftBracketGlow.Depth:Set(function() return pUI.box.leftBracket.Depth() - 1 end)

	LayoutHelpers.AtVerticalCenterIn(pUI.box.leftBracket, pUI.box.panel)
	LayoutHelpers.AtVerticalCenterIn(pUI.box.leftBracketGlow, pUI.box.panel)

	pUI.box.rightGlowTop:SetTexture(UIUtil.UIFile('/game/bracket-right-energy/bracket_bmp_t.dds'))
	pUI.box.rightGlowMiddle:SetTexture(UIUtil.UIFile('/game/bracket-right-energy/bracket_bmp_m.dds'))
	pUI.box.rightGlowBottom:SetTexture(UIUtil.UIFile('/game/bracket-right-energy/bracket_bmp_b.dds'))

	pUI.box.rightGlowTop.Top:Set(function() return pUI.box.Top() + 2 end)
	pUI.box.rightGlowTop.Left:Set(function() return pUI.box.Right() - 12 end)
	pUI.box.rightGlowBottom.Bottom:Set(function() return pUI.box.Bottom() - 2 end)
	pUI.box.rightGlowBottom.Left:Set(pUI.box.rightGlowTop.Left)
	pUI.box.rightGlowMiddle.Top:Set(pUI.box.rightGlowTop.Bottom)
	pUI.box.rightGlowMiddle.Bottom:Set(function() return math.max(pUI.box.rightGlowTop.Bottom(), pUI.box.rightGlowBottom.Top()) end)
	pUI.box.rightGlowMiddle.Right:Set(function() return pUI.box.rightGlowTop.Right() end)

	LayoutHelpers.AtLeftTopIn(pUI.box.title, pUI.box, 15, 10)
	pUI.box.title:SetColor('ffb7e75f')

	LayoutHelpers.AtLeftTopIn(pUI.box.countdown, pUI.box, 15, 41)
	pUI.box.countdown:SetColor('ffb7e75f')		
	
	
	# handicap Special Options
	# Mass
	LayoutHelpers.AtTopIn(pUI.box.mass, pUI.box, 10)
	LayoutHelpers.AtRightIn(pUI.box.mass, pUI.box, 20)
	pUI.box.mass:SetColor('ffb7e75f')
	pUI.box.mass:Hide()

	# Energy
	LayoutHelpers.AtTopIn(pUI.box.energy, pUI.box, 22)
	LayoutHelpers.AtRightIn(pUI.box.energy, pUI.box, 20)
	pUI.box.energy:SetColor('fff7c70f')
	pUI.box.energy:Hide()

	# Kills
	LayoutHelpers.AtTopIn(pUI.box.kills, pUI.box, 14)
	LayoutHelpers.AtRightIn(pUI.box.kills, pUI.box, 50)
	pUI.box.kills:SetColor('fff30017')
	pUI.box.kills:Hide()

	pUI.box.killIcon:SetTexture(UIUtil.UIFile('/game/unit-over/icon-skull_bmp.dds'))
	LayoutHelpers.AtTopIn(pUI.box.killIcon, pUI.box, 11)
	LayoutHelpers.AtRightIn(pUI.box.killIcon, pUI.box, 70)
	pUI.box.killIcon:Hide()

	# Buttons
	pUI.box.massBtn.Width:Set(36)
	pUI.box.massBtn.Height:Set(36)
	LayoutHelpers.AtLeftTopIn(pUI.box.massBtn, pUI.box, 235, 32)
	pUI.box.massBtn.Depth:Set(function() return pUI.box.Depth() + 10 end)
	pUI.box.massBtn:Hide()
	
	pUI.box.energyBtn.Width:Set(36)
	pUI.box.energyBtn.Height:Set(36)
	LayoutHelpers.AtLeftTopIn(pUI.box.energyBtn, pUI.box, 271, 32)
	pUI.box.energyBtn.Depth:Set(function() return pUI.box.Depth() + 10 end)
	pUI.box.energyBtn:Hide()
	
	# Hide panel
	pUI.box:Hide()
	#ShowHideHandicapElements(false)
	pUI.arrow:SetCheck(true, true)
	pUI.box.Left:Set(parent.Left()-pUI.box.Width())

	LOG("Finished HandicapVoting > SetLayout", 0)		
	
end

function CommonLogic()
	# Add heartbeat
	GameMain.AddBeatFunction(HandicapUIBeat)
	GameMain.AddBeatFunction(InitHandicapPanel)
	
	pUI.box.OnDestroy = function(self)
		GameMain.RemoveBeatFunction(HandicapUIBeat)
	end

	# Button Actions
	pUI.arrow.OnCheck = function(self, checked)
		ToggleHandicapPanel()
	end

	LOG("Finished HandicapVoting > CommonLogic", 0)		
	
end

function HandicapUIBeat()
	HandicapSeconds = declare
	if table.getn(pData.handicap_armies) == 0 then
		timeLeft = HandicapSeconds - GetGameTimeSeconds()
		if timeLeft < 0 then timeLeft = 0 end
		if timeLeft == 1 or timeLeft < 1  then
			pUI.box.countdown:SetText("")
		else
			pUI.box.countdown:SetText(string.format("Time until assignment:  %02d:%02d", math.floor(timeLeft / 60), math.mod(timeLeft, 60)))
		end	
	end
	
	# Happening often ~ 100ms
	#LOG("Finished HandicapVoting > HandicapUIBeat", 0)		
	
end

function SetAssignment(assignment)
	pUI.box.title:SetText(assignment)
end

function SetHandicapData(data)
	if voteDialog then
		voteDialog:Destroy()
		voteDialog = nil
	end

	pData.isHandicap = data.isHandicap
	pData.handicap_armies = data.handicap_armies
	pData.innocent_armies = data.innocent_armies
	
	ShowHideHandicapElements(true)
	
	# Remove unnessessary countdown heartbeat
	GameMain.RemoveBeatFunction(HandicapUIBeat)
end

function SetHandicapStats(stats)
	pStats = stats
	# This is no longer needed - to be removed
	
end

function SetHandicapEco(pEco)
	if pData.isHandicap then
		pUI.box.mass:SetText("+" .. pEco.mass)
		pUI.box.energy:SetText("+" .. pEco.energy)
	end
end

function pMassBtnClick()
	data = {
		From = GetFocusArmy(),
		To = -1,
		Name = "HandicapDumpResources",
		Args = "MASS"
	}
	import('/lua/UserPlayerQuery.lua').Query( data, QueryResult )
end

function pEnergyBtnClick()
	data = {
		From = GetFocusArmy(),
		To = -1,
		Name = "HandicapDumpResources",
		Args = "ENERGY"
	}
	import('/lua/UserPlayerQuery.lua').Query( data, QueryResult )	
end

function ToggleHandicapPanel(state)
	if import('/lua/ui/game/gamemain.lua').gameUIHidden and state != nil then
		return
	end

	if UIUtil.GetAnimationPrefs() then
		if state or pUI.box:IsHidden() then
			PlaySound(Sound({Cue = "UI_Score_Window_Open", Bank = "Interface"}))
			pUI.box:Show()
			#ShowHideHandicapElements(true)
			pUI.box:SetNeedsFrameUpdate(true)
			pUI.box.OnFrame = function(self, delta)
				local newLeft = self.Left() + (1000*delta)
				if newLeft > parent.Left()+14 then
					newLeft = parent.Left()+14
					self:SetNeedsFrameUpdate(false)
				end
				self.Left:Set(newLeft)
			end
			pUI.arrow:SetCheck(false, true)
		else
			PlaySound(Sound({Cue = "UI_Score_Window_Close", Bank = "Interface"}))
			pUI.box:SetNeedsFrameUpdate(true)
			pUI.box.OnFrame = function(self, delta)
				local newLeft = self.Left() - (1000*delta)
				if newLeft < parent.Left()-self.Width() then
					newLeft = parent.Left()-self.Width()
					self:SetNeedsFrameUpdate(false)
					self:Hide()
					#ShowHideHandicapElements(false)
				end
				self.Left:Set(newLeft)
			end
			pUI.arrow:SetCheck(true, true)
		end
	else
		if state or pUI.box:IsHidden() then
			pUI.box:Show()
			ShowHideHandicapElements(true)
			pUI.arrow:SetCheck(false, true)
		else
			pUI.box:Hide()
			#ShowHideHandicapElements(false)
			pUI.arrow:SetCheck(true, true)
		end
	end
end

function InitHandicapPanel()
	if import('/lua/ui/game/gamemain.lua').gameUIHidden then
		return
	end
	LOG("InitHandicapPanel", 0)
	
	if UIUtil.GetAnimationPrefs() then
		pUI.box:Show()
		pUI.box:SetNeedsFrameUpdate(true)
		pUI.box.OnFrame = function(self, delta)
			local newLeft = self.Left() + (1000*delta)
			if newLeft > parent.Left()+14 then
				newLeft = parent.Left()+14
				self:SetNeedsFrameUpdate(false)
			end
			self.Left:Set(newLeft)
		end
		pUI.arrow:SetCheck(false, true)

	else
		pUI.box:Show()
		pUI.arrow:SetCheck(false, true)	
	end
	pUI.box.mass:Hide()
	pUI.box.energy:Hide()
	pUI.box.massBtn:Hide()
	pUI.box.energyBtn:Hide()
	pUI.box.kills:Hide()
	pUI.box.killIcon:Hide()
	GameMain.RemoveBeatFunction(InitHandicapPanel)
end

function ShowHideHandicapElements(show) 
	if show then
		if pData.isHandicap then
			pUI.box.mass:Show()
			pUI.box.energy:Show()
			pUI.box.massBtn:Show()
			pUI.box.energyBtn:Show()	
		else
			pUI.box.mass:Hide()
			pUI.box.energy:Hide()
			pUI.box.massBtn:Hide()
			pUI.box.energyBtn:Hide()
		end
		if table.getn(pData.handicap_armies) > 0 then
			pUI.box.kills:Show()
			pUI.box.killIcon:Hide()		
		else
			pUI.box.kills:Hide()
			pUI.box.killIcon:Hide()		
		end
	else
		pUI.box.mass:Hide()
		pUI.box.energy:Hide()
		pUI.box.massBtn:Hide()
		pUI.box.energyBtn:Hide()
		pUI.box.kills:Hide()
		pUI.box.killIcon:Hide()
	end
end

function ShowAlert(args)
	import('/lua/ui/game/announcement.lua').CreateAnnouncement(args[1], pUI.arrow, args[2])
end

function QueryResult(data)
	# Nothing needed
end

function ShowHandicapVote()
	local layout =  UIUtil.SkinnableFile('/mods/'..modPath..'/modules/layout/vote.lua')
	local worldView = import('/lua/ui/game/worldview.lua').view
	
	voteDialog = Bitmap(worldView, UIUtil.SkinnableFile('/dialogs/diplomacy-team-alliance/team-panel_bmp.dds'))
	voteDialog:SetRenderPass(UIUtil.UIRP_PostGlow)  -- just in case our parent is the map
	voteDialog:SetName("Handicap Vote")

	LayoutHelpers.AtCenterIn(voteDialog,worldView)

	local dialogTitle = UIUtil.CreateText(voteDialog, "Handicap Vote", 18, UIUtil.titleFont )
	dialogTitle:SetColor( UIUtil.dialogCaptionColor )
	LayoutHelpers.RelativeTo(dialogTitle, voteDialog, layout, "title", "panel")

	local text = "Select the player you would like to receive the handicap"
	local message = UIUtil.CreateText(voteDialog, text, 12, UIUtil.bodyFont )
	LayoutHelpers.RelativeTo(message, voteDialog, layout, "text", "panel")
	LayoutHelpers.AtHorizontalCenterIn(message,voteDialog)

	local function MakeClickCallback(result)
		return function(self, modifiers)
			voteDialog:Destroy()
			voteDialog = nil
			SetVote(result)
		end
	end

	# How many players are there still alive
	local playercount = 0
	for army, brain in GetArmiesTable().armiesTable do
		LOG("1: ", brain.nickname)

		if brain.nickname == "civilian" then
			LOG("civilian: ", brain.nickname)
		else
			playercount = playercount + 1
		end
	end
	
	# Display the voting buttons
	local playeronesname = string.sub(GetArmiesTable().armiesTable[1].nickname, 1, 9)
	local one = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playeronesname, 12)
	LayoutHelpers.RelativeTo(one, voteDialog, layout, "btn_one", "panel")
	one.OnClick = MakeClickCallback(1)

	local playertwosname = string.sub(GetArmiesTable().armiesTable[2].nickname, 1, 9)	
	local two = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playertwosname, 12)
	LayoutHelpers.RelativeTo(two, voteDialog, layout, "btn_two", "panel")
	two.OnClick = MakeClickCallback(2)

	# Display the correct amount of buttons
	if playercount > 2 then
		local playerthreesname = string.sub(GetArmiesTable().armiesTable[3].nickname, 1, 9)	
		local three = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playerthreesname, 12)
		LayoutHelpers.RelativeTo(three, voteDialog, layout, "btn_three", "panel")
		three.OnClick = MakeClickCallback(3)
	end

	if playercount > 3 then
		local playerfoursname = string.sub(GetArmiesTable().armiesTable[4].nickname, 1, 9)	
		local four = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playerfoursname, 12)
		LayoutHelpers.RelativeTo(four, voteDialog, layout, "btn_four", "panel")
		four.OnClick = MakeClickCallback(4)
	end

	if playercount > 4 then
		local playerfivesname = string.sub(GetArmiesTable().armiesTable[5].nickname, 1, 9)	
		local five = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playerfivesname, 12)
		LayoutHelpers.RelativeTo(five, voteDialog, layout, "btn_five", "panel")
		five.OnClick = MakeClickCallback(5)
	end
	
	if playercount > 5 then
		local playersixsname = string.sub(GetArmiesTable().armiesTable[6].nickname, 1, 9)	
		local six = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playersixsname, 12)
		LayoutHelpers.RelativeTo(six, voteDialog, layout, "btn_six", "panel")
		six.OnClick = MakeClickCallback(6)
	end
	
	if playercount > 6 then
		local playersevensname = string.sub(GetArmiesTable().armiesTable[7].nickname, 1, 9)	
		local seven = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playersevensname, 12)
		LayoutHelpers.RelativeTo(seven, voteDialog, layout, "btn_seven", "panel")
		seven.OnClick = MakeClickCallback(7)
	end
	
	if playercount > 7 then
		local playereightsname = string.sub(GetArmiesTable().armiesTable[8].nickname, 1, 9)	
		local eight = UIUtil.CreateDialogButtonStd(voteDialog, "/dialogs/standard_btn/standard", playereightsname, 12)
		LayoutHelpers.RelativeTo(eight, voteDialog, layout, "btn_eight", "panel")
		eight.OnClick = MakeClickCallback(8)
	end


end

function SetVote(vote)
	data = {
		From = GetFocusArmy(),
		To = -1,
		Name = "HandicapVote",
		Args = { Nick = GetArmiesTable().armiesTable[GetFocusArmy()].nickname, 
				 Vote = vote, 
				 VoteNick = GetArmiesTable().armiesTable[vote].nickname,
				 From = GetFocusArmy() }
	}
	import('/lua/UserPlayerQuery.lua').Query( data, QueryResult )	
end